'' =================================================================================================
''
''   File....... jm_adafruit_ble_rgb_control.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2017 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 23 OCT 2017
''
'' =================================================================================================

{

  Note: CTS must be connected to ground
        Put module into UART mode to experiment with Bluefuit LE app

        Connect to BTLE with Adafruit Bluefruit LE app
        -- select controller
        -- select Color Picker interface

}


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us
  

con { io pins }                                                  
                                                                 
  RX1  = 31  { I }                                              ' serial / programming
  TX1  = 30  { O }                                           
                                                              
  SDA1 = 29  { I/O }                                            ' i2c / eeprom
  SCL1 = 28  { I/O }

  RX2  = 24  { I }                                              ' BTLE
  TX2  = 27  { O }

  LEDS = 15  { O }                                              ' WS2812


con

  STRIP_LEN = 24                                                ' Adafruit 24-pixel ring
  PIX_BITS  = 24                                                ' RGB pixels


obj

' main                                                          ' * master Spin cog
  time  : "jm_time_80"                                          '   timing and delays
  io    : "jm_io_basic"                                         '   essential io
  btle  : "jm_fullduplexserial"                                 ' * serial IO for BTLE module
  strip : "jm_rgbx_pixel"                                       ' * smart pixel driver
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var

  long  pixbuf[STRIP_LEN]                                       ' pixel buffers

  byte  packet[6]                                               ' packet  

                                                                 
pub main | color                                                     
                                                                 
  setup

  time.pause(50)
  btle.rxflush                                                        
                                                                 
  repeat
    if (get_packet == 6)                                        ' correct packet size?
      if (check_packet)                                         ' good packet?
        color := packet[2] << 24                                ' red
        color |= packet[3] << 16                                ' green
        color |= packet[4] <<  8                                ' blue
        strip.set_all(color)


pub get_packet : len | b

'' Wait for packet
'' -- returns # of bytes recieved (up to 6 for color picker)

  bytefill(@packet, 0, 6)                                       ' clear packet buffer

  b := btle.rxcheck                                             ' anything waiting
  if (b <> "!")                                                 ' correct header?
    return 0                                                    ' no -- abort
  else
    packet[len++] := b                                          ' start packet

  b := btle.rxtime(5)                                           ' wait for next byte        
  if (b <> "C")                                                 ' Color picker packet?           
    return 0                                                    ' no -- abort               
  else                      
    packet[len++] := b
    
  repeat 4                                                      ' get colors and checksum
    b := btle.rxtime(5)
    if (b => 0)
      packet[len++] := b
    else
      quit

  return len                                                    ' return length


pub check_packet : cs | idx

'' Returns true if calculated checksum matches received checksum

  repeat idx from 0 to 4
    cs += packet[idx]

  cs := !cs & $FF

  return (cs == packet[5])  

                                  
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)

  btle.start(RX2, TX2, %0000, 9600)                             ' start serial for BTLE  module *

  strip.start_2812b(@pixbuf, STRIP_LEN, LEDS, 0_5)              ' start pixel driver for WS2812b *   
                                                              
                          
dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      